const fs = require("fs");
const InstantiatedSDK = require("./instantiated_node.js");
const http = require("http");

function logTask(type, message) {
    switch(type) {
        case 0:
            console.log(Date(Date.now) + " [INFO]: "+message);
            break;
        case 1:
            console.log(Date(Date.now) + " [WARN]: "+message);
            break;
        case 2:
            console.log(Date(Date.now) + " [ERROR]: "+message);
            process.exit();
            break;
    }
}

logTask(0, "Starting the directory service...");

let loadedProfile = false;
let profileName = "profile";
let profile = {
    id: "directory",
    password: "password",
    maxUsers: 50,
    http_ip: "127.0.0.1",
    http_port: 9990,
    directory: {
        "Image Boards": [],
        "Membership Platforms": [],
        "Health Websites": [],
        "Labratory Websites": [],
        "Domain Registrar": [],
        "Social Media": [],
        "Personal Blogs": [],
        "Video Sharing": [],
        "Video Games": [],
        "MMO Websites": [],
        "Online Stores": [],
        "Recipe Websites": [],
        "Dating Sites": [],
        "Information and News": [],
        "Downloadable Programs": [],
        "Live Streaming": [],
        "Movie and TV Streaming": [],
        "Forums": [],
        "Chatrooms": [],
        "Archives": [],
        "Libraries of Books": [],
        "Operating Systems": [],
        "Tools": [],
        "Hosting": [],
        "Repositories": [],
        "Web Series": [],
        "Entertainment": [],
        "Web Directories": [],
        "Roleplaying": [],
        "Others": [
            {
                "web": "http:\/\/instantiated.xyz\/",
                "site_description": "Instantiated allows the \"User\" to host an instance from one of our Instance Servers hosting in USA. Think of it as your own personal home that you can do whatever you wish to do in it, make any rule you want others to follow, setup a moderation team of your own and announce hosted events on you're own instance ensuring that we bring the power to the user on self moderation systems that will never go away.",
                "site_title": "Instantiated",
                "trust_rating": "safe",
                "note": ""
            }
        ]
    }
};

let webCounter=0;

const args = process.argv;

for(var i=0; i<args.length; i++) {
    switch(args[i]) {
        case "-profile":
            logTask(0, "Loading up '"+args[i+1]+"' profile please wait...");
            loadedProfile=true;
            profileName = args[i+1];
            if(fs.existsSync(args[i+1]+".json")) {
                profile = JSON.parse(fs.readFileSync(args[i + 1]+".json"));
                logTask(0, "Profile loaded up!");
            } else {
                fs.writeFileSync(args[i+1]+".json", JSON.stringify(profile, null, '\t'));
                logTask(1, "Profile couldn't be found so we are creating one from scratch...");
            }
            break;
    }
}

if(!loadedProfile) {
    logTask(0, "Since no profile was specified we are going to load up the default profile.");
    if(fs.existsSync("profile.json")) {
        profile = JSON.parse(fs.readFileSync("profile.json"));
    } else {
        fs.writeFileSync("profile.json", JSON.stringify(profile, null, '\t'));
    }
}

for(p in profile["directory"]) {
    webCounter+=profile["directory"][p].length;
}

const httpServer = http.createServer((req, res) => {
    var query = new URL(req.url, `http://${req.headers.host}`);
    var q = query.searchParams;
    var pathname = query.pathname;

    var html=``;
    var links = ``;
    var contentType = `text/html`;
    switch(pathname) {
        case "/":
            html = `<html><head><title>`+profile["id"]+`'s Internet directory</title></head><body><p><b>Create a new category/Delete category</b><br><form action="/service" method="GET"><input type="text" name="cat" placeholder="Place your category name here..." /> <input type="submit" name="cmd" value="Add Category" /></form></p><b>Review pending websites.</b><br>`;

            // Review websites added.
            for(var website in profile["directory"]) {
                html+=`<b>`+website+`</b><br>`;
                for(var i=0; i<profile["directory"][website].length; i++) {
                    if(profile["directory"][website][i]["trust_rating"] == "pending") {
                        html+=`<p><form action="/service" method="GET">`;
                        html+=`<input type="hidden" name="cat" value="`+website+`" />`;
                        html+=`<input type="hidden" name="id" value="`+i+`" />`;
                        html+=`Website: <a href="`+profile["directory"][website][i]["web"]+`" target="_blank">`+profile["directory"][website][i]["site_title"]+`</a><br>Description: <p>`+profile["directory"][website][i]["site_description"]+`</p><select name="rating"><option value="safe">Safe</option><option value="questionable">Questionable</option><option value="risky">Risky</option><option value="adult_content">Adult Content</option><option value="danger">Dangerous</option><option value="broken">Broken Link</option><option value="offline">Offline</option></select><br>Notes: <input type="text" name="staffnotes" /> <input type="submit" name="cmd" value="Approve Site" /> <input type="submit" name="cmd" value="Delete Site" /></form></p>`;
                    }
                }
            }

            html+=`<br><br><b>All added websites.</b><br>`

            // Websites that were added.
            for(var website in profile["directory"]) {
                html+=`<b>`+website+`</b> - <a href="/service?cmd=Delete Category&cat=`+encodeURIComponent(website)+`">Delete Category</a><br>`;
                for(var i=0; i<profile["directory"][website].length; i++) {
                    if(profile["directory"][website][i]["trust_rating"] != "pending") {
                        html+=`<p><form action="/service?id=`+i+`" method="GET">`;
                        html+=`<input type="hidden" name="cat" value="`+website+` />`;
                        html+=`Website: <a href="`+profile["directory"][website][i]["web"]+`" target="_blank">`+profile["directory"][website][i]["site_title"]+`</a><br>Description: <p>`+profile["directory"][website][i]["site_description"]+`</p><input type="submit" name="cmd" value="Delete Site" /></form></p>`;
                    }
                }
            }
            break;
        case "/service":
            var itemid = parseInt(q.get("id"));
            var web = q.get("cat");
            var cmd = q.get("cmd");
            var notes = q.get("staffnotes");
            var rating = q.get("rating");

            switch(cmd) {
                case "Approve Site":
                    logTask(0, "Site approved, saving to database...");
                    profile["directory"][web][itemid]["trust_rating"] = rating;
                    profile["directory"][web][itemid]["note"] = notes;
                    fs.writeFileSync(profileName+".json", JSON.stringify(profile, null, '\t'));
                    instance.Receive({
                        id: "SiteHasBeenAdded",
                        category: web,
                        website: profile["directory"][web][itemid]
                    });
                    html=`<script>window.location.href="/";</script>`;
                    break;
                case "Delete Site":
                    logTask(0, "Site deleted, saving to database...");
                    profile["directory"][web].splice(itemid, 1);
                    fs.writeFileSync(profileName+".json", JSON.stringify(profile, null, '\t'));
                    instance.Receive({
                        id: "SiteHasBeenRemoved",
                        category: web,
                        website: profile["directory"][web][itemid]
                    });
                    html=`<script>window.location.href="/";</script>`;
                    break;
                case "Add Category":
                    logTask(0, "Creating category...");
                    if(web in profile["directory"]) {
                        logTask(1, "The category already exists with that name. Try again with another name.");
                    } else {
                        profile["directory"][web] = [];
                        fs.writeFileSync(profileName+".json", JSON.stringify(profile, null, '\t'));
                        instance.Receive({
                            id: "CategoryAdded",
                            category: web
                        });
                    }
                    html=`<script>window.location.href="/";</script>`;
                    break;
                case "Delete Category":
                    logTask(0, "Deleting category...");
                    if(web in profile["directory"]) {
                        delete profile["directory"][web];
                        fs.writeFileSync(profileName+".json", JSON.stringify(profile, null, '\t'));
                        instance.Receive({
                            id: "CategoryDeleted",
                            category: web
                        });
                    } else {
                        logTask(1, "Category "+web+" does not exists...");
                    }
                    html=`<script>window.location.href="/";</script>`;
                    break;
            }
    }

    res.writeHead(200, { 'Content-Type': contentType });
    res.end(html);
});

httpServer.on("listening", () => {
    logTask(0, "HTTP Admin Panel is listening on http://"+profile["http_ip"]+":"+profile["http_port"]);
})

httpServer.listen(profile["http_port"], profile["http_ip"]);

let instance = new InstantiatedSDK();
instance.onOpen(() => {
    instance.CreateServer(profile["id"], "Directory Service", profile["maxUsers"], false, profile["password"], false, false);

    setInterval(function() {
        instance.Ping();
    }, 15000);
});

instance.onClose(() => {
    logTask(2, "The server has closed it's connection.");
});

instance.onMessage((data) => {
    var id=JSON.parse(data);

    //console.log(id);

    switch(id["MSG"]) {
        case "SocketID":
            logTask(0, "Assigning socket id to "+id["SocketID"]+"...");
            instance.SetSocketID(id["SocketID"]);
            break;
        case "created_session":
            logTask(0, "Copy and paste this to your URL: https://instantiated.xyz/Samples/WebDirectory/#"+id["SessionGUID"]);
            instance.SetSessionID(id["SessionGUID"]);
            break;
        case "joined_session":
            logTask(0, "Client #"+id["SocketID"]+" has joined with IP="+id["IPAddress"]+", Flags="+JSON.stringify(id["Flags"]));
            // Lets send them the category list.

            var list=[];
            var counts=[];
            for(l in profile["directory"]) {
                list.push(l);
                counts.push(profile["directory"][l].length);
            }

            instance.PrivateSend({
                id: "Categories",
                CatLists: list,
                Counters: counts,
                Total: webCounter
            }, id["SocketID"]);
            break;
        case "Disconnected":
            logTask(0, "Client #"+id["SocketID"]+" has left the session.");
            break;
        case "Tx":
            var tx=id["DataStream"];

            switch(tx["id"]) {
                case "AddSite":
                    logTask(0, "Client #"+id["SocketID"]+":"+id["IPAddress"]+" has submitted a request to add the site "+tx["url"]);
                    var cat = tx["category"];
                    var sitename = tx["name"];
                    var siteurl = tx["url"];
                    var sitedesc = tx["description"];
                    if(tx["category"] in profile["directory"]) {
                        for(var i=0; i<profile["directory"].length; i++) {
                            if(profile["directory"][cat][i]["web"] == siteurl) {
                                logTask(1, "Site "+tx["url"]+" already exists, aborting request.");
                                instance.PrivateSend({
                                    id: "SiteAlreadyAdded",
                                    url: siteurl
                                }, id["SocketID"]);
                                break;
                            }
                        }

                        logTask(0, "Site "+tx["url"]+" has been submitted for owner review.");
                        profile["directory"][cat].push({
                            "web": siteurl,
                            "site_description": sitedesc,
                            "site_title": sitename,
                            "trust_rating": "pending",
                            "note": ""
                        });
                        webCounter++;
                        fs.writeFileSync(profileName+".json", JSON.stringify(profile, null, '\t'));

                        instance.PrivateSend({
                            id: "SiteAddedForReview",
                            category: cat,
                            url: siteurl,
                            description: sitedesc,
                            name: sitename,
                            Total: webCounter
                        }, id["SocketID"])
                    } else {
                        instance.PrivateSend({
                            id: "CategoryNotFound",
                            category: cat
                        }, id["SocketID"]);
                    }
                    break;
                case "ListSites":
                    var cat = tx["category"];
                    var trust = tx["filter_trust"];
                    var data=[];

                    for(var i=0; i<profile["directory"][cat].length; i++) {
                        if(profile["directory"][cat][i]["trust_rating"] == trust || trust == "all") {
                            data.push(Object.assign({index: i}, profile["directory"][cat][i]));
                        }
                    }

                    instance.PrivateSend({
                        id: "WebsiteDatabaseList",
                        database: data
                    }, id["SocketID"]);
                    break;
                case "ListCategories":
                    var list=[];
                    var counts=[];
                    for(l in profile["directory"]) {
                        list.push(l);
                        counts.push(profile["directory"][l].length);
                    }

                    instance.PrivateSend({
                        id: "Categories",
                        CatLists: list,
                        Counters: counts,
                        Total: webCounter
                    }, id["SocketID"]);
                    break;
            }
            break;
    }
});